@@ -1,11 +1,13 @@
 // src/analysis.rs
 
+use crate::limits::enforce_limits;
+use crate::limits::LimitPolicy;
 use crate::models::fers::fers::FERS;
+use nalgebra::DMatrix;
 use serde_json;
 use std::fs;
-use std::path::Path;   
-use std::io;     
-use nalgebra::DMatrix;
+use std::io;
+use std::path::Path;
 
 /// Perform analysis on the given JSON input string.
 ///
@@ -19,27 +21,30 @@ use nalgebra::DMatrix;
 /// Returns an `Ok(String)` with a success message, or an `Err(String)` on failure.
 pub fn calculate_from_json_internal(json_data: &str) -> Result<String, String> {
     // 1) Parse
-    let mut fers: FERS = serde_json::from_str(json_data)
-        .map_err(|e| format!("Bad JSON input: {}", e))?;
+    let mut fers: FERS =
+        serde_json::from_str(json_data).map_err(|e| format!("Bad JSON input: {}", e))?;
+
+    let policy = LimitPolicy::free();
+    enforce_limits(&fers, &policy)?;
 
     // 2) Options
     let opts = &fers.settings.analysis_option;
     let is_second = match opts.order.as_str() {
-        "Linear"    => false,
+        "Linear" => false,
         "Nonlinear" => true,
         other => return Err(format!("Unknown order `{}`", other)),
     };
-    let tol    = opts.tolerance;
+    let tol = opts.tolerance;
     let max_it = opts.max_iterations.unwrap_or(20) as usize;
 
     let mut all_results = Vec::new();
 
     // Collect IDs before any mutable operations
-    let load_case_ids: Vec<_> = fers.load_cases.iter()
-        .map(|lc| lc.id)
-        .collect();
-    
-    let combo_ids: Vec<_> = fers.load_combinations.iter()
+    let load_case_ids: Vec<_> = fers.load_cases.iter().map(|lc| lc.id).collect();
+
+    let combo_ids: Vec<_> = fers
+        .load_combinations
+        .iter()
         .map(|combo| combo.load_combination_id)
         .collect();
 
@@ -58,24 +63,14 @@ pub fn calculate_from_json_internal(json_data: &str) -> Result<String, String> {
     }
 
     // 4) Always solve *all* load‐combinations
-    
+
     for combo_id in combo_ids {
         let res = if is_second {
-            fers.solve_for_load_combination_second_order(
-                combo_id,
-                max_it,
-                tol,
-            )
-            .map_err(|e| format!(
-                "Combo {} 2nd‑order error: {}",
-                combo_id, e
-            ))?
+            fers.solve_for_load_combination_second_order(combo_id, max_it, tol)
+                .map_err(|e| format!("Combo {} 2nd‑order error: {}", combo_id, e))?
         } else {
             fers.solve_for_load_combination(combo_id)
-                .map_err(|e| format!(
-                    "Combo {} linear error: {}",
-                    combo_id, e
-                ))?
+                .map_err(|e| format!("Combo {} linear error: {}", combo_id, e))?
         };
         all_results.push(res);
     }
@@ -89,7 +84,6 @@ pub fn calculate_from_json_internal(json_data: &str) -> Result<String, String> {
     }
 }
 
-
 /// A small helper for printing displacement or reaction vectors more nicely.
 #[allow(dead_code)]
 fn print_readable_vector(vector: &DMatrix<f64>, label: &str) {
@@ -119,7 +113,7 @@ pub fn calculate_from_file_internal(path: &str) -> Result<String, String> {
 
 pub fn load_fers_from_file<P: AsRef<Path>>(path: P) -> Result<FERS, io::Error> {
     let s = fs::read_to_string(path)?;
-    let fers: FERS = serde_json::from_str(&s)
-        .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?;
+    let fers: FERS =
+        serde_json::from_str(&s).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?;
     Ok(fers)
-}
\ No newline at end of file
+}
