@@ -227,38 +227,60 @@ pub fn compute_member_results_from_displacement(
                     continue;
                 }
 
-                // active: compute axial force (tension +)
+                // active: compute axial force (tension > 0)
                 let n = member.calculate_axial_force_3d(
                     global_displacement_vector,
                     &material_map,
                     &section_map,
                 );
 
-                // unit direction (global) from start → end
-                let (dx, dy, dz) = {
-                    let length = member.calculate_length() as f64;
-                    let ux = (member.end_node.X - member.start_node.X) / length;
-                    let uy = (member.end_node.Y - member.start_node.Y) / length;
-                    let uz = (member.end_node.Z - member.start_node.Z) / length;
-                    (ux, uy, uz)
-                };
+                // global unit direction from start → end
+                let dx = member.end_node.X - member.start_node.X;
+                let dy = member.end_node.Y - member.start_node.Y;
+                let dz = member.end_node.Z - member.start_node.Z;
+                let l = (dx * dx + dy * dy + dz * dz).sqrt();
+                if l == 0.0 {
+                    // degenerate; report zeros
+                    let zeros = NodeForces {
+                        fx: 0.0,
+                        fy: 0.0,
+                        fz: 0.0,
+                        mx: 0.0,
+                        my: 0.0,
+                        mz: 0.0,
+                    };
+                    results_by_member_identifier.insert(
+                        member.id,
+                        MemberResult {
+                            start_node_forces: zeros,
+                            end_node_forces: zeros,
+                            maximums: zeros,
+                            minimums: zeros,
+                        },
+                    );
+                    continue;
+                }
+                let ex = dx / l;
+                let ey = dy / l;
+                let ez = dz / l;
 
                 let start = NodeForces {
-                    fx: n * dx,
-                    fy: n * dy,
-                    fz: n * dz,
+                    fx: -n * ex,
+                    fy: -n * ey,
+                    fz: -n * ez,
                     mx: 0.0,
                     my: 0.0,
                     mz: 0.0,
                 };
                 let end = NodeForces {
-                    fx: -n * dx,
-                    fy: -n * dy,
-                    fz: -n * dz,
+                    fx: n * ex,
+                    fy: n * ey,
+                    fz: n * ez,
                     mx: 0.0,
                     my: 0.0,
                     mz: 0.0,
                 };
+
                 let (maximums, minimums) = compute_component_extrema(&start, &end);
 
                 results_by_member_identifier.insert(
@@ -296,7 +318,7 @@ pub fn compute_member_results_from_displacement(
                                 continue;
                             }
 
-                            let length = member.calculate_length() as f64;
+                            let length = member.calculate_length();
 
                             // end intensities (same convention as in assembler)
                             let (w1, w2) = match distributed_load.distribution_shape {
@@ -386,7 +408,7 @@ pub fn compute_member_results_from_displacement(
                                         continue;
                                     }
 
-                                    let length = member.calculate_length() as f64;
+                                    let length = member.calculate_length();
 
                                     let (w1, w2) = match distributed_load.distribution_shape {
                                         crate::models::loads::distributionshape::DistributionShape::Uniform => {
