@@ -1,14 +1,25 @@
+// src/models/fers/fers.rs
+use crate::functions::geometry::{
+    compute_num_dofs_from_members, dof_index, local_to_global_with_releases,
+};
 use crate::functions::results::{compute_member_results_from_displacement, extract_displacements};
-use crate::models::members::memberhinge::{classify_from_hinges, AxisMode, AxisModes};
+use crate::functions::rigid_graph::RigidGraph;
+use crate::functions::support_utils::{add_support_springs_to_operator, constrain_single_dof};
 use nalgebra::DMatrix;
 use serde::{Deserialize, Serialize};
 use std::collections::{BTreeMap, HashMap, HashSet};
 use utoipa::ToSchema;
-// use csv::Writer;
-// use std::error::Error;
+
+use crate::functions::hinge_and_release_operations::{
+    build_local_truss_translational_spring_k, modes_from_single_ends,
+};
+use crate::functions::load_assembler::{
+    assemble_distributed_loads, assemble_nodal_loads, assemble_nodal_moments,
+};
 use crate::functions::reactions::{
     compose_support_reaction_vector_equilibrium, extract_reaction_nodes,
 };
+
 use crate::models::imperfections::imperfectioncase::ImperfectionCase;
 use crate::models::loads::loadcase::LoadCase;
 use crate::models::loads::loadcombination::LoadCombination;
@@ -24,14 +35,6 @@ use crate::models::settings::settings::Settings;
 use crate::models::supports::nodalsupport::NodalSupport;
 use crate::models::supports::supportconditiontype::SupportConditionType;
 
-use crate::functions::hinge_and_release_operations::{
-    apply_end_releases_to_local_beam_k, build_local_truss_translational_spring_k,
-    modes_from_single_ends,
-};
-use crate::functions::load_assembler::{
-    assemble_distributed_loads, assemble_nodal_loads, assemble_nodal_moments,
-};
-
 #[derive(Serialize, Deserialize, ToSchema, Debug)]
 pub struct FERS {
     pub member_sets: Vec<MemberSet>,
@@ -73,67 +76,10 @@ impl<'a> AssemblyContext<'a> {
     }
 }
 
-const TRANSLATION_AXES: [(&str, usize); 3] = [("X", 0), ("Y", 1), ("Z", 2)];
-const ROTATION_AXES: [(&str, usize); 3] = [("X", 3), ("Y", 4), ("Z", 5)];
-
-macro_rules! get_case_insensitive {
-    ($map:expr, $key:expr) => {
-        $map.get($key)
-            .or_else(|| $map.get(&$key.to_ascii_lowercase()))
-    };
-}
+pub const TRANSLATION_AXES: [(&str, usize); 3] = [("X", 0), ("Y", 1), ("Z", 2)];
+pub const ROTATION_AXES: [(&str, usize); 3] = [("X", 3), ("Y", 4), ("Z", 5)];
 
 impl FERS {
-    fn visit_unique_supported_nodes<F>(&self, mut visitor: F) -> Result<(), String>
-    where
-        F: FnMut(u32, usize, &NodalSupport) -> Result<(), String>,
-    {
-        use std::collections::{HashMap, HashSet};
-
-        let support_by_id: HashMap<u32, &NodalSupport> =
-            self.nodal_supports.iter().map(|s| (s.id, s)).collect();
-
-        let mut visited_node_ids: HashSet<u32> = HashSet::new();
-
-        for member_set in &self.member_sets {
-            for member in &member_set.members {
-                for node in [&member.start_node, &member.end_node] {
-                    if !visited_node_ids.insert(node.id) {
-                        continue;
-                    }
-                    if let Some(support_id) = node.nodal_support {
-                        if let Some(nodal_support) = support_by_id.get(&support_id) {
-                            let base_index = (node.id as usize - 1) * 6;
-                            visitor(node.id, base_index, *nodal_support)?; // propagate
-                        }
-                    }
-                }
-            }
-        }
-        Ok(())
-    }
-
-    fn spring_stiffness_or_error(
-        owner_id: u32,
-        axis_label: &str,
-        kind_label: &str,
-        stiffness: Option<f64>,
-    ) -> Result<f64, String> {
-        let k = stiffness.ok_or_else(|| {
-            format!(
-                "Support {} {} {} is Spring but stiffness is missing.",
-                owner_id, kind_label, axis_label
-            )
-        })?;
-        if k <= 0.0 {
-            return Err(format!(
-                "Support {} {} {} Spring stiffness must be positive.",
-                owner_id, kind_label, axis_label
-            ));
-        }
-        Ok(k)
-    }
-
     pub fn build_lookup_maps(
         &self,
     ) -> (
@@ -157,50 +103,6 @@ impl FERS {
         (material_map, section_map, memberhinge_map, support_map)
     }
 
-    fn add_support_springs_to_operator(
-        &self,
-        global_stiffness_matrix: &mut nalgebra::DMatrix<f64>,
-    ) -> Result<(), String> {
-        self.visit_unique_supported_nodes(|_node_id, base_index, support| {
-            // Translational springs
-            for (axis_label, local_dof) in TRANSLATION_AXES {
-                if let Some(condition) =
-                    get_case_insensitive!(support.displacement_conditions, axis_label)
-                {
-                    if let SupportConditionType::Spring = condition.condition_type {
-                        let stiffness = Self::spring_stiffness_or_error(
-                            support.id,
-                            axis_label,
-                            "displacement",
-                            condition.stiffness,
-                        )?;
-                        global_stiffness_matrix
-                            [(base_index + local_dof, base_index + local_dof)] += stiffness;
-                    }
-                }
-            }
-
-            // Rotational springs
-            for (axis_label, local_dof) in ROTATION_AXES {
-                if let Some(condition) =
-                    get_case_insensitive!(support.rotation_conditions, axis_label)
-                {
-                    if let SupportConditionType::Spring = condition.condition_type {
-                        let stiffness = Self::spring_stiffness_or_error(
-                            support.id,
-                            axis_label,
-                            "rotation",
-                            condition.stiffness,
-                        )?;
-                        global_stiffness_matrix
-                            [(base_index + local_dof, base_index + local_dof)] += stiffness;
-                    }
-                }
-            }
-            Ok(())
-        })
-    }
-
     fn build_operator_with_supports(
         &self,
         active_map: &std::collections::HashMap<u32, bool>,
@@ -211,98 +113,37 @@ impl FERS {
             let k_geo = self.assemble_geometric_stiffness_matrix_with_active(u, active_map)?;
             k += k_geo;
         }
-        self.add_support_springs_to_operator(&mut k)?;
+        add_support_springs_to_operator(&self.member_sets, &self.nodal_supports, &mut k)?;
         Ok(k)
     }
 
     fn build_rigid_elimination_partial_using_hinges(&self) -> Result<RigidElimination, String> {
-        use crate::models::members::enums::MemberType;
-        use nalgebra::DMatrix;
         use std::collections::{HashMap, HashSet};
 
-        let assembly_context = AssemblyContext::new(self);
-
-        // parent: slave -> (master, r = x_b - x_a)
-        let mut parent: HashMap<u32, (u32, (f64, f64, f64))> = HashMap::new();
-
         #[derive(Clone, Copy)]
         struct RigidInfo {
-            a: u32, // master node id
-            b: u32, // slave node id
+            a: u32,
+            b: u32,
             r: (f64, f64, f64),
-            modes: AxisModes,
         }
+
+        let rigid = RigidGraph::build(&self.member_sets)?;
         let mut rigid_elems: Vec<RigidInfo> = Vec::new();
-
-        for set in &self.member_sets {
-            for m in &set.members {
-                if !matches!(m.member_type, MemberType::Rigid) {
-                    continue;
-                }
-
-                let a = m.start_node.id; // master
-                let b = m.end_node.id; // slave
-
-                let r = (
-                    m.end_node.X - m.start_node.X,
-                    m.end_node.Y - m.start_node.Y,
-                    m.end_node.Z - m.start_node.Z,
-                );
-                let l2 = r.0 * r.0 + r.1 * r.1 + r.2 * r.2;
-                if l2 < 1.0e-24 {
-                    return Err(format!("Rigid member {} has zero length.", m.id));
-                }
-
-                // unique master per slave
-                if parent.contains_key(&b) {
-                    return Err(format!("Node {} is slave in multiple rigid links.", b));
-                }
-
-                // cycle check: does 'a' depend on 'b' upstream?
-                let mut p = a;
-                let mut guard = 0usize;
-                while let Some(&(pp, _)) = parent.get(&p) {
-                    if pp == b {
-                        return Err(format!("Rigid cycle detected involving node {}.", b));
-                    }
-                    p = pp;
-                    guard += 1;
-                    if guard > 100000 {
-                        return Err("Rigid chain too long (suspected loop).".to_string());
-                    }
-                }
-
-                parent.insert(b, (a, r));
-
-                // hinge modes, then FORCE translations to rigid for a rigid link
-                let a_h = m
-                    .start_hinge
-                    .and_then(|id| assembly_context.hinge_by_id.get(&id).copied());
-                let b_h = m
-                    .end_hinge
-                    .and_then(|id| assembly_context.hinge_by_id.get(&id).copied());
-                let mut modes = classify_from_hinges(a_h, b_h);
-
-                // IMPORTANT: translations MUST be rigid for a rigid link
-                modes.trans = [AxisMode::Rigid, AxisMode::Rigid, AxisMode::Rigid];
-
-                rigid_elems.push(RigidInfo { a, b, r, modes });
-            }
+        for e in rigid.edges_sorted_master_first() {
+            rigid_elems.push(RigidInfo {
+                a: e.master,
+                b: e.slave,
+                r: e.r,
+            });
         }
 
-        // number of full DOFs
-        let n_full = self.compute_num_dofs();
+        let n_full = compute_num_dofs_from_members(&self.member_sets);
 
-        // which full dofs are eliminated
+        // eliminate all 6 slave DOFs
         let mut eliminated: HashSet<usize> = HashSet::new();
         for info in &rigid_elems {
-            for ax in 0..3 {
-                if matches!(info.modes.trans[ax], AxisMode::Rigid) {
-                    eliminated.insert(FERS::dof_index(info.b, ax));
-                }
-                if matches!(info.modes.rot[ax], AxisMode::Rigid) {
-                    eliminated.insert(FERS::dof_index(info.b, 3 + ax));
-                }
+            for d in 0..6 {
+                eliminated.insert(dof_index(info.b, d));
             }
         }
 
@@ -314,7 +155,7 @@ impl FERS {
             for m in &set.members {
                 for node in [&m.start_node, &m.end_node] {
                     for d in 0..6 {
-                        let fi = FERS::dof_index(node.id, d);
+                        let fi = dof_index(node.id, d);
                         if eliminated.contains(&fi) {
                             continue;
                         }
@@ -330,48 +171,17 @@ impl FERS {
         let n_red = red_to_full.len();
         let mut s = DMatrix::<f64>::zeros(n_full, n_red);
 
-        // 1) identity for retained dofs
         for (fi, &col) in &full_to_red {
             s[(*fi, col)] = 1.0;
         }
 
-        // depth helper so that masters are processed before their slaves in a chain
-        fn depth_of(node: u32, parent: &HashMap<u32, (u32, (f64, f64, f64))>) -> usize {
-            let mut d = 0usize;
-            let mut p = node;
-            while let Some(&(pp, _)) = parent.get(&p) {
-                d += 1;
-                p = pp;
-            }
-            d
-        }
-
-        // 2) write slave rows by composing master rows via C(r)
-        let mut edges = rigid_elems.clone();
-        edges.sort_by_key(|e| depth_of(e.a, &parent)); // masters before slaves
-
-        for info in &edges {
+        // fill slave rows: [u_b;θ_b] = C(r)[u_a;θ_a]
+        for info in &rigid_elems {
             let c = FERS::rigid_map_c(info.r.0, info.r.1, info.r.2);
-
             for i in 0..6 {
-                let is_trans = i < 3;
-                let ax = if is_trans { i } else { i - 3 };
-                let rigid_here = if is_trans {
-                    matches!(info.modes.trans[ax], AxisMode::Rigid)
-                } else {
-                    matches!(info.modes.rot[ax], AxisMode::Rigid)
-                };
-                if !rigid_here {
-                    continue;
-                }
-
-                let row_b = FERS::dof_index(info.b, i);
-
-                // Zero the row_b to avoid accidental accumulation if multiple edges ever touched it
-
-                // S[row_b, :] = Σ_j C[i,j] * S[row_a_j, :]
+                let row_b = dof_index(info.b, i);
                 for j in 0..6 {
-                    let row_a_j = FERS::dof_index(info.a, j);
+                    let row_a_j = dof_index(info.a, j);
                     let coeff = c[(i, j)];
                     if coeff == 0.0 {
                         continue;
@@ -417,7 +227,7 @@ impl FERS {
         self.validate_node_ids()?;
         let assembly_context = AssemblyContext::new(self);
 
-        let number_of_dofs: usize = self.compute_num_dofs();
+        let number_of_dofs: usize = compute_num_dofs_from_members(&self.member_sets);
 
         let mut global_stiffness_matrix =
             nalgebra::DMatrix::<f64>::zeros(number_of_dofs, number_of_dofs);
@@ -444,27 +254,13 @@ impl FERS {
                                 format!("Member {} failed to build local stiffness.", member.id)
                             })?;
 
-                        // 2) Per-end modes from hinges (LOCAL axes)
-                        let a_h = member
-                            .start_hinge
-                            .and_then(|id| assembly_context.hinge_by_id.get(&id).copied());
-                        let b_h = member
-                            .end_hinge
-                            .and_then(|id| assembly_context.hinge_by_id.get(&id).copied());
-                        let (a_trans, a_rot, b_trans, b_rot) = modes_from_single_ends(a_h, b_h);
-
-                        // 3) Apply releases & semi-rigid springs (condensation in LOCAL)
-                        let k_local_mod = apply_end_releases_to_local_beam_k(
+                        // 2) Apply end releases / semi-rigid springs and transform to GLOBAL in one place
+                        let k_global = local_to_global_with_releases(
+                            member,
                             &k_local_base,
-                            a_trans,
-                            a_rot,
-                            b_trans,
-                            b_rot,
+                            &assembly_context.hinge_by_id,
                         )?;
 
-                        // 4) Transform to GLOBAL
-                        let t = member.calculate_transformation_matrix_3d();
-                        let k_global = t.transpose() * k_local_mod * t;
                         Some(k_global)
                     }
 
@@ -543,7 +339,7 @@ impl FERS {
     ) -> Result<nalgebra::DMatrix<f64>, String> {
         use crate::models::members::enums::MemberType;
         let assembly_context: AssemblyContext<'_> = AssemblyContext::new(self);
-        let n = self.compute_num_dofs();
+        let n = compute_num_dofs_from_members(&self.member_sets);
         let mut k_geo = nalgebra::DMatrix::<f64>::zeros(n, n);
 
         for member_set in &self.member_sets {
@@ -568,27 +364,12 @@ impl FERS {
                 );
                 let k_g_local_base = member.calculate_geometric_stiffness_matrix_3d(n_axial);
 
-                // Apply same end releases/semi-rigid springs as for material stiffness
-                let a_h = member
-                    .start_hinge
-                    .and_then(|id| assembly_context.hinge_by_id.get(&id).copied());
-                let b_h = member
-                    .end_hinge
-                    .and_then(|id| assembly_context.hinge_by_id.get(&id).copied());
-                let (a_trans, a_rot, b_trans, b_rot) = modes_from_single_ends(a_h, b_h);
-
-                let k_g_local_mod = apply_end_releases_to_local_beam_k(
+                let k_g_global = local_to_global_with_releases(
+                    member,
                     &k_g_local_base,
-                    a_trans,
-                    a_rot,
-                    b_trans,
-                    b_rot,
+                    &assembly_context.hinge_by_id,
                 )?;
 
-                // Transform to GLOBAL
-                let t = member.calculate_transformation_matrix_3d();
-                let k_g_global = t.transpose() * k_g_local_mod * t;
-
                 let i0 = (member.start_node.id as usize - 1) * 6;
                 let j0 = (member.end_node.id as usize - 1) * 6;
                 Self::assemble_element_into_global_12(&mut k_geo, i0, j0, &k_g_global);
@@ -668,22 +449,11 @@ impl FERS {
         changed
     }
 
-    fn compute_num_dofs(&self) -> usize {
-        let max_node = self
-            .member_sets
-            .iter()
-            .flat_map(|ms| ms.members.iter())
-            .flat_map(|m| vec![m.start_node.id, m.end_node.id])
-            .max()
-            .unwrap_or(0) as usize;
-        max_node * 6
-    }
-
     pub fn assemble_load_vector_for_combination(
         &self,
         combination_id: u32,
     ) -> Result<DMatrix<f64>, String> {
-        let num_dofs = self.compute_num_dofs();
+        let num_dofs = compute_num_dofs_from_members(&self.member_sets);
         let mut f_comb = DMatrix::<f64>::zeros(num_dofs, 1);
 
         // Find the combination by its load_combination_id field
@@ -702,10 +472,6 @@ impl FERS {
         Ok(f_comb)
     }
 
-    fn dof_index(node_id: u32, local_dof: usize) -> usize {
-        (node_id as usize - 1) * 6 + local_dof
-    }
-
     fn rigid_map_c(r_x: f64, r_y: f64, r_z: f64) -> nalgebra::SMatrix<f64, 6, 6> {
         use nalgebra::{Matrix3, SMatrix};
 
@@ -734,23 +500,6 @@ impl FERS {
         &elim.s * u_red
     }
 
-    fn constrain_single_dof(
-        &self,
-        k_global: &mut DMatrix<f64>,
-        rhs: &mut DMatrix<f64>,
-        dof_index: usize,
-        prescribed: f64,
-    ) {
-        for j in 0..k_global.ncols() {
-            k_global[(dof_index, j)] = 0.0;
-        }
-        for i in 0..k_global.nrows() {
-            k_global[(i, dof_index)] = 0.0;
-        }
-        k_global[(dof_index, dof_index)] = 1.0;
-        rhs[(dof_index, 0)] = prescribed;
-    }
-
     fn constrain_linear_constraint_penalty(
         &self,
         k_red: &mut nalgebra::DMatrix<f64>,
@@ -853,9 +602,6 @@ impl FERS {
         k_red: &mut nalgebra::DMatrix<f64>,
         rhs_red: &mut nalgebra::DMatrix<f64>,
     ) -> Result<(), String> {
-        use crate::models::supports::supportconditiontype::SupportConditionType;
-        use std::collections::HashMap;
-
         let support_map: HashMap<u32, &NodalSupport> =
             self.nodal_supports.iter().map(|s| (s.id, s)).collect();
 
@@ -882,7 +628,7 @@ impl FERS {
                         continue;
                     };
 
-                    let base_full: usize = (node.id as usize - 1) * 6;
+                    let base_full = (node.id as usize - 1) * 6;
 
                     // Handle translational constraints
                     for (axis_label, local_dof) in TRANSLATION_AXES {
@@ -902,7 +648,7 @@ impl FERS {
                         let fi: usize = base_full + local_dof;
                         if let Some(ri) = elim.full_to_red.get(&fi).copied() {
                             // Retained DOF: constrain exactly
-                            self.constrain_single_dof(k_red, rhs_red, ri, 0.0);
+                            constrain_single_dof(k_red, rhs_red, ri, 0.0);
                         } else {
                             // Slave DOF: enforce (S_row * u_red) = 0
                             let mut a_column = nalgebra::DMatrix::<f64>::zeros(n_red, 1);
@@ -914,7 +660,7 @@ impl FERS {
                             if let Some(pivot_j) =
                                 self.detect_one_hot_constraint(&a_column, ONE_HOT_TOL_RATIO)
                             {
-                                self.constrain_single_dof(k_red, rhs_red, pivot_j, 0.0);
+                                constrain_single_dof(k_red, rhs_red, pivot_j, 0.0);
                             } else {
                                 self.constrain_linear_constraint_penalty(
                                     k_red,
@@ -944,7 +690,7 @@ impl FERS {
                         let fi: usize = base_full + local_dof;
                         if let Some(ri) = elim.full_to_red.get(&fi).copied() {
                             // Retained rotational DOF: constrain exactly
-                            self.constrain_single_dof(k_red, rhs_red, ri, 0.0);
+                            constrain_single_dof(k_red, rhs_red, ri, 0.0);
                         } else {
                             // Slave rotational DOF: enforce (S_row * u_red) = 0
                             let mut a_column = nalgebra::DMatrix::<f64>::zeros(n_red, 1);
@@ -955,7 +701,7 @@ impl FERS {
                             if let Some(pivot_j) =
                                 self.detect_one_hot_constraint(&a_column, ONE_HOT_TOL_RATIO)
                             {
-                                self.constrain_single_dof(k_red, rhs_red, pivot_j, 0.0);
+                                constrain_single_dof(k_red, rhs_red, pivot_j, 0.0);
                             } else {
                                 self.constrain_linear_constraint_penalty(
                                     k_red,
@@ -975,7 +721,7 @@ impl FERS {
     }
 
     pub fn assemble_load_vector_for_case(&self, load_case_id: u32) -> DMatrix<f64> {
-        let num_dofs = self.compute_num_dofs();
+        let num_dofs = compute_num_dofs_from_members(&self.member_sets);
         let mut f = DMatrix::<f64>::zeros(num_dofs, 1);
 
         if let Some(load_case) = self.load_cases.iter().find(|lc| lc.id == load_case_id) {
@@ -1001,8 +747,6 @@ impl FERS {
         map
     }
 
-    // Do not abbreviate code
-
     fn solve_first_order_common(
         &mut self,
         load_vector_full: nalgebra::DMatrix<f64>,
@@ -1014,7 +758,8 @@ impl FERS {
         let axial_slack_tolerance: f64 = AXIAL_SLACK_TOLERANCE_DEFAULT;
 
         let mut active_map = self.init_active_map_tie_comp();
-        let mut u_full = nalgebra::DMatrix::<f64>::zeros(self.compute_num_dofs(), 1);
+        let mut u_full =
+            nalgebra::DMatrix::<f64>::zeros(compute_num_dofs_from_members(&self.member_sets), 1);
 
         let assembly_context: AssemblyContext<'_> = AssemblyContext::new(self);
 
@@ -1079,6 +824,7 @@ impl FERS {
             Some(&active_map), // <- so ties/struts respect active set
         )?;
 
+        // Store masked reactions
         let mut sum_rx = 0.0;
         let mut sum_ry = 0.0;
         let mut sum_rz = 0.0;
@@ -1142,7 +888,7 @@ impl FERS {
         let axial_slack_tolerance: f64 = AXIAL_SLACK_TOLERANCE_DEFAULT;
 
         let mut active_map = self.init_active_map_tie_comp();
-        let n_full = self.compute_num_dofs();
+        let n_full = compute_num_dofs_from_members(&self.member_sets);
         let mut u_full = nalgebra::DMatrix::<f64>::zeros(n_full, 1);
 
         let assembly_context: AssemblyContext<'_> = AssemblyContext::new(self);
@@ -1273,7 +1019,6 @@ impl FERS {
             ResultType::Loadcombination(combination_id),
         )
     }
-
     pub fn solve_for_load_combination_second_order(
         &mut self,
         combination_id: u32,
