@@ -1,8 +1,7 @@
 use serde::{Deserialize, Serialize};
 use utoipa::ToSchema;
 
-
-#[derive(Serialize, Deserialize, ToSchema, Clone, Debug)]
+#[derive(Serialize, Deserialize, ToSchema, Clone, Copy, Debug)]
 pub struct NodeForces {
     pub fx: f64,
     pub fy: f64,
@@ -10,4 +9,4 @@ pub struct NodeForces {
     pub mx: f64,
     pub my: f64,
     pub mz: f64,
-} 
\ No newline at end of file
+}
